@@ -10,7 +10,7 @@ class NetBoxInventoryConfig(PluginConfig):
     author = 'Matej Vadnjal'
     author_email = 'matej.vadnjal@arnes.si'
     base_url = 'inventory'
-    min_version = '3.4.0'
+    min_version = '3.5.2'
     default_settings = {
         'top_level_menu': True,
         'used_status_name': 'used',
@@ -20,8 +20,13 @@ class NetBoxInventoryConfig(PluginConfig):
         'asset_import_create_device_type': False,
         'asset_import_create_module_type': False,
         'asset_import_create_inventoryitem_type': False,
+        'asset_import_create_tenant': False,
         'asset_disable_editing_fields_for_tags': {},
         'asset_disable_deletion_for_tags': [],
+        'asset_custom_fields_search_filters': {},
+        'asset_warranty_expire_warning_days': 90,
+        'prefill_asset_name_create_inventoryitem': False,
+        'prefill_asset_tag_create_inventoryitem': False,
     }
 
     def ready(self):
