@@ -4,13 +4,14 @@ from django.utils.text import slugify
 
 from dcim.models import DeviceType, Manufacturer, ModuleType, Location, Site
 from netbox.forms import NetBoxModelBulkEditForm, NetBoxModelImportForm
-from utilities.forms import (
-    add_blank_choice, ChoiceField, CommentField, CSVChoiceField,
-    CSVModelChoiceField, DynamicModelChoiceField
+from utilities.forms import add_blank_choice
+from utilities.forms.fields import (
+    ChoiceField, CommentField, CSVChoiceField, CSVModelChoiceField,
+    DynamicModelChoiceField
 )
-from tenancy.models import Tenant
+from tenancy.models import Contact, Tenant
 from ..choices import AssetStatusChoices, HardwareKindChoices
-from ..models import Asset, InventoryItemType, InventoryItemGroup, Purchase, Supplier
+from ..models import Asset, Delivery, InventoryItemType, InventoryItemGroup, Purchase, Supplier
 from ..utils import get_plugin_setting
 
 __all__ = (
@@ -20,6 +21,8 @@ __all__ = (
     'SupplierBulkEditForm',
     'PurchaseImportForm',
     'PurchaseBulkEditForm',
+    'DeliveryImportForm',
+    'DeliveryBulkEditForm',
     'InventoryItemTypeImportForm',
     'InventoryItemTypeBulkEditForm',
     'InventoryItemGroupImportForm',
@@ -66,12 +69,10 @@ class AssetBulkEditForm(NetBoxModelBulkEditForm):
         help_text=Asset._meta.get_field('purchase').help_text,
         required=not Asset._meta.get_field('purchase').blank,
     )
-    order_number = forms.CharField(
-        required=False,
-    )
-    # FIXME figure out how to use DateFicker field for these
-    purchase_date = forms.CharField(
-        required=False,
+    delivery = DynamicModelChoiceField(
+        queryset=Delivery.objects.all(),
+        help_text=Asset._meta.get_field('delivery').help_text,
+        required=not Asset._meta.get_field('delivery').blank,
     )
     warranty_start = forms.CharField(
         required=False,
@@ -79,6 +80,16 @@ class AssetBulkEditForm(NetBoxModelBulkEditForm):
     warranty_end = forms.CharField(
         required=False,
     )
+    tenant = DynamicModelChoiceField(
+        queryset=Tenant.objects.all(),
+        help_text=Asset._meta.get_field('tenant').help_text,
+        required=not Asset._meta.get_field('tenant').blank,
+    )
+    contact = DynamicModelChoiceField(
+        queryset=Contact.objects.all(),
+        help_text=Asset._meta.get_field('contact').help_text,
+        required=not Asset._meta.get_field('contact').blank,
+    )
     storage_location = DynamicModelChoiceField(
         queryset=Location.objects.all(),
         help_text=Asset._meta.get_field('storage_location').help_text,
@@ -92,12 +103,13 @@ class AssetBulkEditForm(NetBoxModelBulkEditForm):
     fieldsets = (
         ('General', ('name', 'status')),
         ('Hardware', ('device_type', 'device', 'module_type', 'module')),
-        ('Purchase', ('owner', 'purchase', 'warranty_start', 'warranty_end')), 
+        ('Purchase', ('owner', 'purchase', 'delivery', 'warranty_start', 'warranty_end')), 
+        ('Assigned to', ('tenant', 'contact')), 
         ('Location', ('storage_location',)),
     )
     nullable_fields = (
-        'name', 'device', 'module', 'owner', 'purchase', 'warranty_start',
-        'warranty_end',
+        'name', 'device', 'module', 'owner', 'purchase', 'delivery', 'tenant', 'contact',
+        'warranty_start', 'warranty_end',
     )
 
 
@@ -115,7 +127,7 @@ class AssetImportForm(NetBoxModelImportForm):
     )
     model_name = forms.CharField(
         required=True,
-        help_text='Model of this device/module/inventory item type. See "Import settings" below for more info.',
+        help_text='Model of this device/module/inventory item type. See "Import settings" for more info.',
     )
     part_number = forms.CharField(
         required=False,
@@ -147,10 +159,22 @@ class AssetImportForm(NetBoxModelImportForm):
         help_text='Tenant that owns this asset. It must exist before import.',
         required=False,
     )
-    purchase = CSVModelChoiceField(
-        queryset=Purchase.objects.all(),
+    delivery = forms.CharField(
+        help_text='Delivery that delivered this asset. See "Import settings" for more info.',
+        required=False,
+    )
+    delivery_date = forms.DateField(
+        help_text='Date when this delivery was made.',
+        required=False,
+    )
+    receiving_contact = CSVModelChoiceField(
+        queryset=Contact.objects.all(),
         to_field_name='name',
-        help_text='Purchase through which this asset was purchased. See "Import settings" below for more info.',
+        help_text='Contact that accepted this delivery. It must exist before import.',
+        required=False,
+    )
+    purchase = forms.CharField(
+        help_text='Purchase through which this asset was purchased. See "Import settings" for more info.',
         required=False,
     )
     purchase_date = forms.DateField(
@@ -160,7 +184,19 @@ class AssetImportForm(NetBoxModelImportForm):
     supplier = CSVModelChoiceField(
         queryset=Supplier.objects.all(),
         to_field_name='name',
-        help_text='Legal entity this purchase was made from. Required if a new purchase was given.',
+        help_text='Legal entity this purchase was made from. Required if a new purchase is given.',
+        required=False,
+    )
+    tenant = CSVModelChoiceField(
+        queryset=Tenant.objects.all(),
+        to_field_name='name',
+        help_text='Tenant using this asset. See "Import settings" for more info.',
+        required=False,
+    )
+    contact = CSVModelChoiceField(
+        queryset=Contact.objects.all(),
+        to_field_name='name',
+        help_text='Contact using this asset. It must exist before import.',
         required=False,
     )
 
@@ -170,8 +206,9 @@ class AssetImportForm(NetBoxModelImportForm):
             'name', 'asset_tag', 'serial', 'status',
             'hardware_kind', 'manufacturer', 'model_name', 'part_number',
             'model_comments', 'storage_site', 'storage_location',
-            'owner', 'purchase', 'purchase_date', 'supplier',
-            'warranty_start', 'warranty_end', 'comments',
+            'owner', 'supplier', 'purchase', 'purchase_date', 'delivery',
+            'delivery_date', 'receiving_contact', 'warranty_start',
+            'warranty_end', 'comments', 'tenant', 'contact', 'tags',
         )
 
     def clean_model_name(self):
@@ -194,6 +231,28 @@ class AssetImportForm(NetBoxModelImportForm):
         setattr(self.instance, f'{hardware_kind}_type', hardware_type)
         return hardware_type
 
+    def clean_purchase(self):
+        supplier = self.cleaned_data.get('supplier')
+        purchase_name = self.cleaned_data.get('purchase')
+        if not purchase_name:
+            return None
+        try:
+            purchase = Purchase.objects.get(supplier=supplier, name=purchase_name)
+        except ObjectDoesNotExist:
+            raise forms.ValidationError(f'Unable to find purchase {supplier} {purchase_name}')
+        return purchase
+
+    def clean_delivery(self):
+        purchase = self.cleaned_data.get('purchase')
+        delivery_name = self.cleaned_data.get('delivery')
+        if not delivery_name:
+            return None
+        try:
+            delivery = Delivery.objects.get(purchase=purchase, name=delivery_name)
+        except ObjectDoesNotExist:
+            raise forms.ValidationError(f'Unable to find delivery {purchase} {delivery_name}')
+        return delivery
+
     def __init__(self, data=None, *args, **kwargs):
         super().__init__(data, *args, **kwargs)
 
@@ -236,16 +295,25 @@ class AssetImportForm(NetBoxModelImportForm):
             # handle creating related resources if they don't exist and enabled in settings
             if (get_plugin_setting('asset_import_create_purchase')
                 and self.data.get('purchase') and self.data.get('supplier')):
-                Purchase.objects.get_or_create(
+                purchase, _ = Purchase.objects.get_or_create(
                     name=self.data.get('purchase'),
-                    supplier=self._get_or_create_supplier(),
+                    supplier=self._get_or_create_related('supplier'),
                     defaults={'date': self._get_clean_value('purchase_date')}
                 )
+                if self.data.get('delivery'):
+                    Delivery.objects.get_or_create(
+                        name=self.data.get('delivery'),
+                        purchase=purchase,
+                        defaults={
+                            'date': self._get_clean_value('delivery_date'),
+                            'receiving_contact': self._get_clean_value('receiving_contact'),
+                        }
+                    )
             if (get_plugin_setting('asset_import_create_device_type')
                 and self.data.get('hardware_kind') == 'device'):
                 DeviceType.objects.get_or_create(
                     model__iexact=self.data.get('model_name'),
-                    manufacturer=self._get_or_create_manufacturer(),
+                    manufacturer=self._get_or_create_related('manufacturer'),
                     defaults={
                         'model': self.data.get('model_name'),
                         'slug': slugify(self.data.get('model_name')),
@@ -257,7 +325,7 @@ class AssetImportForm(NetBoxModelImportForm):
                 and self.data.get('hardware_kind') == 'module'):
                 ModuleType.objects.get_or_create(
                     model__iexact=self.data.get('model_name'),
-                    manufacturer=self._get_or_create_manufacturer(),
+                    manufacturer=self._get_or_create_related('manufacturer'),
                     defaults={
                         'model': self.data.get('model_name'),
                         'part_number': self._get_clean_value('part_number'),
@@ -268,7 +336,7 @@ class AssetImportForm(NetBoxModelImportForm):
                 and self.data.get('hardware_kind') == 'inventoryitem'):
                 InventoryItemType.objects.get_or_create(
                     model__iexact=self.data.get('model_name'),
-                    manufacturer=self._get_or_create_manufacturer(),
+                    manufacturer=self._get_or_create_related('manufacturer'),
                     defaults={
                         'model': self.data.get('model_name'),
                         'slug': slugify(self.data.get('model_name')),
@@ -276,6 +344,12 @@ class AssetImportForm(NetBoxModelImportForm):
                         'comments': self._get_clean_value('model_comments'),
                     },
                 )
+            if (get_plugin_setting('asset_import_create_tenant')
+                and self.data.get('tenant')):
+                self._get_or_create_related('tenant')
+            if (get_plugin_setting('asset_import_create_tenant')
+                and self.data.get('owner')):
+                self._get_or_create_related('owner')
         except forms.ValidationError as e:
             # ValidationErrors are raised by _clean_fields() method
             # this will be called later in the code and will be bound
@@ -285,25 +359,27 @@ class AssetImportForm(NetBoxModelImportForm):
             # any other errors we add to non-field specific form errors
             self.add_error(None, e)
 
-    def _get_or_create_manufacturer(self):
-        manufacturer, _ = Manufacturer.objects.get_or_create(
-            name__iexact=self.data.get('manufacturer'),
-            defaults={
-                'name': self.data.get('manufacturer'),
-                'slug': slugify(self.data.get('manufacturer'))
-            },
+    def _get_or_create_related(self, field_name):
+        """
+        Create instance of related object if it doesn't exist.
+        Supports specifiying related object by name or slug.
+        """
+        klass = self.fields[field_name].queryset.model
+        # user could have specified alternative field (tenant name or tenant slug)
+        to_field_name = self.fields[field_name].to_field_name
+        # create sensible default data if we'll need to create object
+        instance_defaults = {
+            'name': self.data.get(field_name),
+            'slug': slugify(self.data.get(field_name)),
+        }
+        # whatever field was in import data is used as is
+        instance_defaults.update({to_field_name: self.data.get(field_name)})
+        instance, _ = klass.objects.get_or_create(
+            # filter on field specified in column header
+            **{to_field_name+'__iexact':self.data.get(field_name)},
+            defaults=instance_defaults
         )
-        return manufacturer
-
-    def _get_or_create_supplier(self):
-        supplier, _ = Supplier.objects.get_or_create(
-            name__iexact=self.data.get('supplier'),
-            defaults={
-                'name': self.data.get('supplier'),
-                'slug': slugify(self.data.get('supplier'))
-            }
-        )
-        return supplier
+        return instance
 
     def _get_clean_value(self, field_name):
         """
@@ -314,7 +390,7 @@ class AssetImportForm(NetBoxModelImportForm):
         instance does not exist it raises Exception but no feedback is given to user.
         """
         try:
-            return self.fields[field_name].clean(self.data.get(field_name))
+            return self.base_fields[field_name].clean(self.data.get(field_name))
         except forms.ValidationError as e:
             self.add_error(field_name, e)
             raise
@@ -347,7 +423,7 @@ class PurchaseImportForm(NetBoxModelImportForm):
     supplier = CSVModelChoiceField(
         queryset=Supplier.objects.all(),
         to_field_name='name',
-        help_text=' Legal entity this purchase was made at. It must exist when importing.',
+        help_text='Legal entity this purchase was made at. It must exist when importing.',
         required=True,
     )
     
@@ -381,6 +457,55 @@ class PurchaseBulkEditForm(NetBoxModelBulkEditForm):
     nullable_fields = ('date', 'description',)
 
 
+class DeliveryImportForm(NetBoxModelImportForm):
+    purchase = CSVModelChoiceField(
+        queryset=Purchase.objects.all(),
+        to_field_name='id',
+        help_text='Purchase that this delivery is part of. It must exist when importing.',
+        required=True,
+    )
+    receiving_contact = CSVModelChoiceField(
+        queryset=Contact.objects.all(),
+        to_field_name='id',
+        help_text='Contact that accepted this delivery. It must exist when importing.',
+        required=False,
+    )
+    
+    class Meta:
+        model = Delivery
+        fields = (
+            'name', 'date', 'purchase', 'receiving_contact', 'description', 'comments', 'tags'
+        )
+
+
+class DeliveryBulkEditForm(NetBoxModelBulkEditForm):
+    date = forms.CharField(
+        required=False,
+    )
+    purchase = DynamicModelChoiceField(
+        queryset=Purchase.objects.all(),
+        required=False,
+        label='Purchase',
+    )
+    receiving_contact = DynamicModelChoiceField(
+        queryset=Contact.objects.all(),
+        required=False,
+        label='Receiving Contact',
+    )
+    description = forms.CharField(
+        required=False,
+    )
+    comments = CommentField(
+        required=False,
+    )
+
+    model = Delivery
+    fieldsets = (
+        ('General', ('date', 'purchase', 'receiving_contact', 'description',)),
+    )
+    nullable_fields = ('date', 'description', 'receiving_contact',)
+
+
 class InventoryItemTypeImportForm(NetBoxModelImportForm):
     manufacturer = CSVModelChoiceField(
         queryset=Manufacturer.objects.all(),
