@@ -7,23 +7,6 @@ from dcim.models import Device, Module, InventoryItem
 from .choices import AssetStatusChoices
 
 
-def get_asset_warranty_context(asset):
-    warranty_progress = None
-    bar_class = 'bg-success'
-    if asset.warranty_elapsed and asset.warranty_elapsed.total_seconds() > 0 and asset.warranty_total:
-        warranty_progress = asset.warranty_elapsed / asset.warranty_total
-    if warranty_progress is not None:
-        warranty_progress = warranty_progress * 100
-        if asset.warranty_remaining.days < 7:
-            bar_class = 'bg-danger'
-        elif asset.warranty_remaining.days < 30:
-            bar_class = 'bg-warning'
-    return dict(
-        warranty_progress=warranty_progress,
-        bar_class=bar_class
-    )
-
-
 def get_prechange_field(obj, field_name):
     """Get value from obj._prechange_snapshot. If field is a relation,
     return object instance.
@@ -103,6 +86,36 @@ def asset_clear_old_hw(old_hw):
     pre_save.connect(prevent_update_serial_asset_tag, sender=InventoryItem)
 
 
+def asset_set_new_hw(asset, hw):
+    """
+    Asset was assigned to hardware (device/module/inventory item) and we want to
+    sync some field values from asset to hardware
+    Validation if asset can be assigned to hw should be done before calling this function.
+    """
+    # device, module... needs None for blank asset_tag to enforce uniqness at DB level
+    new_asset_tag = asset.asset_tag or None
+    # device, module... does not allow serial to be null
+    new_serial = asset.serial or ''
+    hw_save = False
+    if hw.serial != new_serial:
+        hw.serial = new_serial
+        hw_save = True
+    if hw.asset_tag != new_asset_tag:
+        hw.asset_tag = new_asset_tag
+        hw_save = True
+    # for inventory items also set manufacturer and part_number
+    if asset.inventoryitem_type:
+        if hw.manufacturer != asset.inventoryitem_type.manufacturer:
+            hw.manufacturer = asset.inventoryitem_type.manufacturer
+            hw_save = True
+        part_id = asset.inventoryitem_type.part_number or asset.inventoryitem_type.model
+        if hw.part_id != part_id:
+            hw.part_id = part_id
+            hw_save = True
+    if hw_save:
+        hw.save()
+
+
 def is_equal_none(a, b):
     """ Compare a and b as string. None is considered the same as empty string. """
     if a is None or b is None:
@@ -148,3 +161,21 @@ def query_located(queryset, field_name, values, assets_shown='all'):
     else:
         raise Exception('unsupported')
     return queryset.filter(q)
+
+
+def get_asset_custom_fields_search_filters():
+    """Returns a list of custom field filter strings that can be used in Q() filter.
+
+    Custom fields and filters are used is defined in the plugin configuration,
+    under the key ``asset_custom_fields_search_filters``.
+
+    Returns:
+        list: list of custom field filter strings
+    """
+    custom_fields_filters = get_plugin_setting('asset_custom_fields_search_filters')
+
+    fields = []
+    for field_name, filters in custom_fields_filters.items():
+        for filter in filters:
+            fields.append(f"custom_field_data__{field_name}__{filter}")
+    return fields
